use itertools::Itertools;

use crate::arg;
use crate::assembler;

#[derive(Debug)]
pub enum Node {
    Comment(String),
    Label(String),
    Call { name: String, arg: Option<arg::Arg> },
    MacroCall { name: String, args: Vec<arg::Arg> },
}

impl assembler::ToAssembly for Node {
    fn to_assembly(&self) -> String {
        match self {
            Node::Comment(x) => format!("; {x}"),
            Node::Label(x) => format!("{x}:"),
            Node::Call { name, arg } => {
                if let Some(a) = arg {
                    format!("{name} {arg}", arg = a.to_assembly())
                } else {
                    name.clone()
                }
            }
            Node::MacroCall { name, args } => {
                if args.is_empty() {
                    format!(".{name}")
                } else {
                    format!(
                        ".{name} {}",
                        args.iter().map(|a| a.to_assembly()).join(", ")
                    )
                }
            }
        }
    }
}

#[derive(Debug)]
pub struct AST {
    pub body: Vec<Node>,
}

impl assembler::ToAssembly for AST {
    fn to_assembly(&self) -> String {
        self.body.iter().map(|n| n.to_assembly()).join("\n")
    }
}
